/*
 * 	Doku, siehe http://www.mikrocontroller.net/articles/AVR_FAT32
 *  Neuste Version: http://www.mikrocontroller.net/svnbrowser/avr-fat32/
 *	Autor: Daniel R.
 */

#include <stdlib.h>
#include <avr/io.h>

#include "config.h"	// Hier werden alle nötigen Konfigurationen vorgenommen, umbedingt anschauen !
#include "file.h"
#include "fat.h"
#include "mmc.h"		// Hardware abhängig
#include "uart.h"		// Hardware abhängig, es kann auch eine eigene eingebunden werden !


//*****************************************************************************************************************
void main(void){

	// Uart initialisierung zu Ausgabe von Daten
	uinit();

	uputs((unsigned char*)"\nBoot");

	// Versuch Karte zu Initialisieren, bis es klappt. Umbedingt so, weil die Initialiesierung nicht immer auf anhieb klappt.
	while (FALSE==mmc_init()){
		nop();
	}

	uputs((unsigned char*)"...");


	// Fat initialisieren. Nur wenn das klappt sind weitere Aktionen sinnvoll, sonst aendet das Programm !
	if(TRUE == fat_loadFatData()){

	  uputs((unsigned char*)"Ok\n");		// Wenn auf dem terminal "Boot... OK" zu lesen ist, ist Init OK. Jetzt kann man schreiben/anhaengen/lesen

	  // In der config.h kann eingestellt werden, ob mit oder ohne lange Dateinamen Unterstuetzung kompiliert wird !
	  #if(MMC_LFN_SUPPORT==TRUE)
		  unsigned char file_name[]="test.txt";
	  #else
		  unsigned char file_name[]="TEST    TXT";
	  #endif

	  // String zum in die Datei schreiben.
	  unsigned char str[]="Hallo Datei!";




	  // Nur wenn schreiben Option in der config.h TRUE ist wird geschrieben bzw. angehangen!
	  #if (MMC_WRITE==TRUE)

		  // Datei existiert nicht, also anlegen !
		  if(MMC_FILE_NEW == ffopen(file_name)){

			 // Schreibt String auf Karte ! Nur richtige Strings koennen mit ffwrites geschrieben werden !
			 ffwrites(str);

			 // Neue Zeile in der Datei
			 ffwrite(0x0D);
			 ffwrite(0x0A);

			 // Schließt Datei
			 ffclose();
		   }




		   // Datei existiert, also anhaengen !
		  if(MMC_FILE_EXISTS == ffopen(file_name)){

			 ffseek(file.length);   // Spult bis zum Dateiende vor um anzuhaengen, geht auch ohne Option MMC_OVER_WRITE !

			 // schreibt String
			 ffwrites(str);

			 // neue Zeile in der Datei
			 ffwrite(0x0D);
			 ffwrite(0x0A);

			 // schließt Datei
			 ffclose();
		   }

	   #endif


	   // Datei existiert also lesen. Gerade angelegt und beschrieben, oder war schon vorhanden und es wurde was angehaengt.
	   if(MMC_FILE_EXISTS == ffopen(file_name)){

	      // setzen einer Variable und dann runterzaehlen geht am schnellsten !
	      unsigned long int seek=file.length;

	      // lesen eines chars und Ausgabe des chars.
	      // solange bis komplette Datei gelesen wurde.
	      do{
	         uputc(ffread());
	      }while(--seek);

	      ffclose();
	   }




	  }

}
